package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.visitor.AttributModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.AttributeModifierVisitorReturnException;
import de.fhdw.wtf.common.token.Token;

/**
 * 
 * Represents a "mutable"-Attributemodifier. The containing attribute is changeable after codegeneration.
 */
public final class AttributeModifierMutable extends AttributeModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 8739729865194347077L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 4747;
	
	/**
	 * Private Constructor of {@link AttributeModifierMutable}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private AttributeModifierMutable(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link AttributeModifierMutable}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return The AttributeModifierMutable-Object.
	 */
	public static AttributeModifierMutable create(final Token firstToken, final Token lastToken) {
		return new AttributeModifierMutable(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link AttributeModifierMutable}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @return The AttributeModifierMutable-Object.
	 */
	public static AttributeModifierMutable create(final Token firstToken) {
		return new AttributeModifierMutable(firstToken, null);
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof AttributeModifierMutable;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	@Override
	public String toString() {
		return de.fhdw.wtf.common.constants.parser.AstDescriptionConstants.MODIFIER_MUTABLE;
	}
	
	@Override
	public boolean accept(final AttributModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final AttributeModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final AttributeModifierVisitorReturnException<X, Y> visitor) throws Y {
		return visitor.handle(this);
	}
}
