package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.ast.visitor.OperationModifierVisitor;
import de.fhdw.wtf.common.ast.visitor.OperationModifierVisitorReturn;
import de.fhdw.wtf.common.token.Token;

/**
 * A concrete {@link OperationModifier}. A Operation with this modifier will generated as an abstract operation.
 */
public final class OperationModifierAbstract extends OperationModifier {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 6172586972454009907L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 248932489;
	
	/**
	 * Constructor of {@link OperationModifierAbstract}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	private OperationModifierAbstract(final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link OperationModifierAbstract}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @return The OperationModifierAbstract-Object
	 */
	public static OperationModifierAbstract create(final Token firstToken, final Token lastToken) {
		return new OperationModifierAbstract(firstToken, lastToken);
	}
	
	/**
	 * Creates a {@link OperationModifierAbstract}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @return The OperationModifierAbstract-Object
	 */
	public static OperationModifierAbstract create(final Token firstToken) {
		return new OperationModifierAbstract(firstToken, null);
	}
	
	@Override
	public String toString() {
		return de.fhdw.wtf.common.constants.parser.AstDescriptionConstants.MODIFIER_ABSTRACT;
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof OperationModifierAbstract;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	@Override
	public boolean accept(final OperationModifierVisitor visitor) {
		return visitor.handle(this);
	}
	
	@Override
	public <X> X accept(final OperationModifierVisitorReturn<X> visitor) {
		return visitor.handle(this);
	}
	
}
