package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.DatabaseIDState;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitor;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitorException;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.TypeProxyStateVisitorReturnException;
import de.fhdw.wtf.common.exception.ast.ASTException;

/**
 * This class represents an invalid state of a {@link TypeProxy}. That means, that the referencer did not find a
 * matching definition type for the applied type name.
 */
public final class InvalidState implements TypeProxyState {
	
	/**
	 * The Name, which doesn´t match to any Type.
	 */
	private final Name name;
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 6346411093596490006L;
	
	/**
	 * Constant to define hashCode().
	 */
	private static final int HASHCODE_CONSTANT = 389735;
	
	/**
	 * Constructor of {@link InvalidState}.
	 * 
	 * @param name
	 *            The Name, which doesn´t match to any Type.
	 */
	private InvalidState(final Name name) {
		this.name = name;
	}
	
	/**
	 * Creates a {@link InvalidState}-Object.
	 * 
	 * @param name
	 *            The Name, which doesn´t match to any Type.
	 * @return The InvalidState-Object
	 */
	public static InvalidState create(final Name name) {
		return new InvalidState(name);
	}
	
	@Override
	public boolean equals(final Object o) {
		return o instanceof InvalidState;
	}
	
	@Override
	public int hashCode() {
		return HASHCODE_CONSTANT;
	}
	
	@Override
	public void accept(final TypeProxyStateVisitor v) {
		v.handle(this);
	}
	
	@Override
	public <X> X accept(final TypeProxyStateVisitorReturn<X> v) {
		return v.handle(this);
	}
	
	@Override
	public <Y extends Exception> void accept(final TypeProxyStateVisitorException<Y> v) throws Y {
		v.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final TypeProxyStateVisitorReturnException<X, Y> v) throws Y {
		return v.handle(this);
	}
	
	@Override
	public String toString() {
		return InvalidState.class.getSimpleName();
	}
	
	@Override
	public String getTypeString() {
		return this.toString();
	}
	
	/**
	 * Returns the name, which doesn´t match to any Type.
	 * 
	 * @return name
	 */
	public Name getName() {
		return this.name;
	}
	
	@Override
	public int compareTo(final TypeProxyState o) {
		return o.accept(new TypeProxyStateVisitorReturn<Integer>() {
			
			@Override
			public Integer handle(final ByNameState byName) {
				return -1;
			}
			
			@Override
			public Integer handle(final InvalidState invalidState) {
				return 0;
			}
			
			@Override
			public Integer handle(final ByReferenceState byType) {
				return -1;
			}
		});
	}
	
	@Override
	public DatabaseIDState getTypeId() {
		throw new ASTException("TypeProxy: cannot get database ID when referencing no type");
	}
	
	@Override
	public void setTypeId(final DatabaseIDState typeId) {
		throw new ASTException("TypeProxy: cannot set database ID when referencing no type");
	}
	
}
