package de.fhdw.wtf.common.ast.type;

import de.fhdw.wtf.common.ast.visitor.AtomicTypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitor;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorException;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.CompositeTypeVisitorReturnException;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorReturn;
import de.fhdw.wtf.common.constants.parser.AstDescriptionConstants;
import de.fhdw.wtf.common.exception.ast.ASTException;
import de.fhdw.wtf.common.exception.parser.SumIsAnythingException;
import de.fhdw.wtf.common.token.Token;

/**
 * A {@link ListType} is an ordered {@link CollectionType} of a type.
 */
public final class ListType extends CollectionType {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 5530952595398445798L;
	
	/**
	 * Constructor of {@link ListType}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param isOf
	 *            List of...
	 * @param lastToken
	 *            lastToken
	 */
	private ListType(final Token firstToken, final Type isOf, final Token lastToken) {
		super(firstToken, isOf, lastToken);
	}
	
	/**
	 * Creates a {@link ListType}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param isOf
	 *            List of...
	 * @param lastToken
	 *            lastToken
	 * @return The {@link ListType}-Object
	 */
	public static ListType create(final Token firstToken, final Type isOf, final Token lastToken) {
		return new ListType(firstToken, isOf, lastToken);
	}
	
	/**
	 * Creates a {@link ListType}-Object.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param isOf
	 *            List of...
	 * @return The {@link ListType}-Object
	 */
	public static ListType create(final Token firstToken, final Type isOf) {
		return new ListType(firstToken, isOf, null);
	}
	
	@Override
	public int compareTo(final Type o) {
		return o.accept(new TypeVisitorReturn<Integer>() {
			
			@Override
			public Integer handle(final AtomicType s) {
				return s.accept(new AtomicTypeVisitorReturn<Integer>() {
					
					@Override
					public Integer handle(final BaseType baseType) {
						return 1;
					}
					
					@Override
					public Integer handle(final ClassType clazz) {
						return -1;
					}
				});
			}
			
			@Override
			public Integer handle(final CompositeType c) {
				return c.accept(new CompositeTypeVisitorReturn<Integer>() {
					
					@Override
					public Integer handle(final ListType list) {
						final Type other = list.getOf();
						return ListType.this.getOf().compareTo(other);
					}
					
					@Override
					public Integer handle(final MapType map) {
						return -1;
					}
					
					@Override
					public Integer handle(final ProductType product) {
						return -1;
					}
					
					@Override
					public Integer handle(final SumType sum) {
						return -1;
					}
					
					@Override
					public Integer handle(final ThrownType thrownType) {
						return 1;
					}
				});
			}
			
			@Override
			public Integer handle(final TypeProxy s) {
				return 1;
			}
		});
	}
	
	@Override
	public String toString() {
		return this.getOf().toString() + AstDescriptionConstants.ASTERISK_TOKEN;
	}
	
	@Override
	public String getTypeString() {
		return this.getOf().getTypeString() + AstDescriptionConstants.ASTERISK_TOKEN;
	}
	
	@Override
	public boolean equals(final Object o) {
		if (o instanceof ListType) {
			return this.getOf().equals(((ListType) o).getOf());
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return this.getOf().hashCode();
	}
	
	@Override
	public void accept(final CompositeTypeVisitor v) {
		v.handle(this);
	}
	
	@Override
	public <X> X accept(final CompositeTypeVisitorReturn<X> v) {
		return v.handle(this);
	}
	
	@Override
	public <X extends Exception> void accept(final CompositeTypeVisitorException<X> v) throws X {
		v.handle(this);
		
	}
	
	@Override
	public <X, Y extends Exception> X accept(final CompositeTypeVisitorReturnException<X, Y> v) throws Y {
		return v.handle(this);
	}
	
	@Override
	public boolean isSupertypeOf(final Type type) {
		return false;
	}
	
	@Override
	public ListType getPrototype() {
		return (ListType) super.getPrototype();
	}
	
	@Override
	public void setPrototype(final Type prototype) {
		if (!(prototype instanceof ListType)) {
			throw new ASTException("The prototype \"" + prototype.getClass() + "\" does not match this type \""
					+ this.getClass() + "\"!");
		}
		super.setPrototype(prototype);
	}
	
	@Override
	public Type standardize() throws SumIsAnythingException {
		return this;
	}
	
	@Override
	public boolean isThrownType() {
		return false;
	}
	
}
