package de.fhdw.wtf.common.token.keywords;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the visitable-modifier as a token in WTF.
 */
public final class VisitableToken extends ClassModifierToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -3531898671252509410L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 67438935;
	
	/**
	 * Defines constant "visitable" for {@link VisitableToken}.
	 */
	private static final String TEXTUAL_REPRESENTATION = "visitable";
	
	/**
	 * Private constructor for {@link VisitableToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the {@link ClassModifierToken} <prior> in the original input.
	 */
	private VisitableToken(final Position position) {
		super(position);
	}
	
	/**
	 * Factory method for {@link VisitableToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link ClassModifierToken} in the original input.
	 * @return a new instance of this {@link ClassModifierToken}.
	 */
	public static VisitableToken create(final Position position) {
		return new VisitableToken(position);
	}
	
	@Override
	public boolean isVisitableToken() {
		return true;
	}
	
	@Override
	public int getLength() {
		return VisitableToken.TEXTUAL_REPRESENTATION.length();
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t instanceof VisitableToken;
	}
	
	@Override
	public String stringRepresentation() {
		return VisitableToken.TEXTUAL_REPRESENTATION;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
