package de.fhdw.wtf.common.token.symbols;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the asterisk symbol '*' as a token.
 */
public final class AsteriskToken extends SymbolToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 651607100283044637L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 97345789;
	
	/**
	 * Private constructor for {@link AsteriskToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the symbol '*' in the original input.
	 */
	private AsteriskToken(final Position position) {
		super(position, SymbolConstants.ASTERISK);
	}
	
	/**
	 * Factory method for {@link AsteriskToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link SymbolToken} in the original input.
	 * @return a new instance of this {@link SymbolToken}.
	 */
	public static AsteriskToken create(final Position position) {
		return new AsteriskToken(position);
	}
	
	@Override
	public boolean isAsteriskToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isAsteriskToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymbolConstants.ASTERISK;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
