package de.fhdw.wtf.common.token.symbols;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the equal symbol '=' as a token.
 */
public final class EqualToken extends SymbolToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -1355954616131141780L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 1237987;
	
	/**
	 * Private constructor for {@link EqualToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the symbol '=' in the original input.
	 */
	private EqualToken(final Position position) {
		super(position, SymbolConstants.EQUALS);
	}
	
	/**
	 * Factory method for {@link EqualToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link SymbolToken} in the original input.
	 * @return a new instance of this {@link SymbolToken}.
	 */
	public static EqualToken create(final Position position) {
		return new EqualToken(position);
	}
	
	@Override
	public boolean isEqualToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isEqualToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymbolConstants.EQUALS;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
}
