package de.fhdw.wtf.parser;

import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.exception.parser.NoIdentifierException;
import de.fhdw.wtf.common.stream.TokenStream;

/**
 * Parser to parse the given TokenStream to {@link Name}s.
 */
final class NameParser {
	
	/**
	 * TokenStream.
	 */
	private final TokenStream stream;
	
	/**
	 * Constructor of {@link NameParser}.
	 * 
	 * @param stream
	 *            tokenStream
	 */
	private NameParser(final TokenStream stream) {
		this.stream = stream;
	}
	
	/**
	 * FactoryMethod for {@link NameParser}.
	 * 
	 * @param stream
	 *            tokenStream
	 * @return The {@link NameParser}-Object.
	 */
	static NameParser createNameParser(final TokenStream stream) {
		return new NameParser(stream);
	}
	
	/**
	 * Parses the next series of Token of stream to Name.
	 * 
	 * @return Name
	 * @throws NoIdentifierException
	 *             NoIdentifierException
	 */
	Name parse() throws NoIdentifierException {
		Name newName = UnqualifiedName.create(ParserUtils.requireAndRemoveIdentifier(this.stream), this.stream.peek());
		
		while (this.stream.hasNext() && this.stream.peek().isGreaterSymbolToken()) {
			this.stream.removeFirst();
			newName = newName.addName(ParserUtils.requireAndRemoveIdentifier(this.stream));
		}
		return newName;
	}
	
}
